# type of the spans
TYPE = 'redis'

# net extension
DB = 'out.redis_db'

# standard tags
RAWCMD = 'redis.raw_command'
ARGS_LEN = 'redis.args_length'
PIPELINE_LEN = 'redis.pipeline_length'
PIPELINE_AGE = 'redis.pipeline_age'
IMMEDIATE_PIPELINE = 'redis.pipeline_immediate_command'
